# Vagrant CFSSL

This project permits to build from scratch and in few minutes a running **CFSSL** in a virtual machine on your laptop.

## Prerequisites

On your computer you need to have installed :
* [Vagrant](https://www.vagrantup.com/downloads)
* [VirtualBox](https://www.virtualbox.org/)
* Git
* a Ruby version management tool like RVM or [RBenv](https://github.com/rbenv/rbenv)
* r10k (`gem install r10k`)
* Internet access
* Bash

For Linux users (debian, ubuntu), do *not* install Vagrant and VirtualBox from `apt-get` (packages are way too old),
download `.deb` from the respective websites.

---------------------------------

## Setup documentation

Three steps to get you own running CFSSL on your laptop.

### Step 1 - Clone this repository
```bash
git clone https://gitlab.adullact.net/adullact/pki/vagrant-(...)
cd vagrant-(...)
```

### Step 2 - Download all required Puppet modules used to configure the virtual machine
```bash
./BUILD.sh
```

### Step 3 - Boot a virtual machine, download and configure all required items to get a running CFSSL

Some configuration values are presents in file `hieradata/parameters.yaml`. You can modify this file 
to change value of any parameter documented by Puppet component module `cfssl` in 
[REFERENCE.md](https://gitlab.adullact.net/adullact/puppet-cfssl/-/blob/main/REFERENCE.md)

```bash
vagrant up
```
Then you wait few minutes (depends on your network access and power of your computer).

---------------------------------

## Usage documentation

1. Obtain public certificate of certification authority (CA)
2. Generate a new private key and certificate
3. **TODO** Get information about a certificate
4. **TODO** Revoke a certificate

### 1. Get public certificate of certification authority (CA)

[CFSSL documentation, API endpoint `/api/v1/cfssl/info`](https://github.com/cloudflare/cfssl/blob/master/doc/api/endpoint_info.txt)

```bash
# /api/v1/cfssl/info
# ---> Get information about certification authority (CA), including the CA certificate
curl -s       \
     -d '{}'  \
     -H "Content-Type: application/json"  \
     -X POST 127.0.0.1:8888/api/v1/cfssl/info | jq
     
# /api/v1/cfssl/info
# ---> Get public certificate of certification authority (CA)
curl -s       \
     -d '{}'  \
     -H "Content-Type: application/json"  \
     -X POST 127.0.0.1:8888/api/v1/cfssl/info | jq -r '.result' | jq -r '.certificate'
```

Example of an API response:
```json
{
  "success": true,
  "result": {
    "certificate": "-----BEGIN CERTIFICATE-----\nMIIDuDCCAqCgAwIBAgIUeYCALLsE4HkgGtrG+/lFFUK65gEwDQYJKoZIhvcNAQEL\nBQAwXjELMAkGA1UEBhMCRlIxFDASBgNVBAcTC01PTlRQRUxMSUVSMREwDwYDVQQK\nEwhBRFVMTEFDVDEmMCQGA1UEAxMdVkFHUkFOVCBBRFVMTEFDVCBST09UIENBIEdF\nTjQwHhcNMjIxMTMwMDA0NDAwWhcNMjUxMTI5MDA0NDAwWjBmMQswCQYDVQQGEwJG\nUjEUMBIGA1UEBxMLTU9OVFBFTExJRVIxETAPBgNVBAoTCEFEVUxMQUNUMS4wLAYD\nVQQDEyVWQUdSQU5UIEFEVUxMQUNUIElOVEVSTUVESUFURSBDQSBHRU4xMIIBIjAN\nBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA1Ne4s8rWSRP6M2nUrC4OaJl/Fdrv\nKvcgileIbLL8kgISQ53QZh7dxdQR8XnjcI3RHS0kwUZmsTj2UrXz+O5jr+N/0kv6\nHHqG9orltcK9qkuGXQFxiZjLDmva9z6MH02ZlqO+3sfI/NNKzqI1ajZLsvMMGclm\neCNhph2YbeOr4kK90qmeL/3NSzLIAyZCXdQkupOW3plzCSnWohuru8TdUTr0pyt0\nt1MDV3p2wYjPEIgzUPQ9z+qaCx1caXRMOLkKwBwAep3ColBoxEq3A+eTMXGToJ/m\ncD4idRLrT5Sr7PrZvFNuQuMK0w0FzUL69g64uD1c66SAGNrzvaKaB8RBtwIDAQAB\no2YwZDAOBgNVHQ8BAf8EBAMCAQYwEgYDVR0TAQH/BAgwBgEB/wIBATAdBgNVHQ4E\nFgQUMQkkiAQG/CgBOTMODyMoZsxfDMEwHwYDVR0jBBgwFoAUVZH9RbrYSgB7Kwjb\n/kGHl0SgIK0wDQYJKoZIhvcNAQELBQADggEBAKrp2v3biB1j4hGFtMmla1dj6Hd9\nPfhJCpU3SiYxaQEY7EeLBSju8GxQE1wuDeh7iq1IxkTBk+3nLhRVY81vBdwHByce\nu3o6Ex+E03qXrIS4PIx/PjSBpPf6QZF9oeXtbOFxWwuTlZ+sa4aLz1ZJHqaB6eah\nHOgt7Cwd3v1rIEHDDDdgf9duebfyz7dFzqAjIoxK+0mABTgpBVFkbj/KAdwQPWEd\nmoiuiGM4tLQUG8X5CCwViz0MlWguWCgGxlMQd/S1Wdgt3M7jU2DBcmK7iOMGmpgp\njBrmYqYlvQQ4JuQlqvbJCPLcv4P8dlyQlFD5B7jm4Rx2CJSn14XdY+Y/vck=\n-----END CERTIFICATE-----",
    "usages": [
      "client auth"
    ],
    "expiry": "8760h"
  },
  "errors": [],
  "messages": []
}

```

### 2. Generate a new private key and certificate

[CFSSL documentation, API endpoint `/api/v1/cfssl/newcert`](https://github.com/cloudflare/cfssl/blob/master/doc/api/endpoint_newcert.txt)

```bash
# /api/v1/cfssl/newcert
# ---> Generate a new private key and certificate
REQUEST='{ "request": {"hosts": [""], "names":[{"C":"FR", "ST":"Herault", "L":"Montpellier", "O":"Mairie de Montpellier", "OU": "465146154365143651"}], "CN": "tintin@montpellier.fr"} }' 
curl -s                \
     -d  "${REQUEST}"  \
     -H "Content-Type: application/json"  \
     -X POST 127.0.0.1:8888/api/v1/cfssl/newcert | jq
```

Example of an API response:
```json
{
  "success": true,
  "result": {
    "certificate": "-----BEGIN CERTIFICATE-----\nMIIDYTCCAkmgAwIBAgIUWKU4fXHUUAbyAbuLWpdJw/ZsYAIwDQYJKoZIhvcNAQEL\nBQAwZjELMAkGA1UEBhMCRlIxFDASBgNVBAcTC01PTlRQRUxMSUVSMREwDwYDVQQK\nEwhBRFVMTEFDVDEuMCwGA1UEAxMlVkFHUkFOVCBBRFVMTEFDVCBJTlRFUk1FRElB\nVEUgQ0EgR0VOMTAeFw0yMjExMzAwMTU5MDBaFw0yMzExMzAwMTU5MDBaMIGSMQsw\nCQYDVQQGEwJGUjEQMA4GA1UECBMHSGVyYXVsdDEUMBIGA1UEBxMLTW9udHBlbGxp\nZXIxHjAcBgNVBAoTFU1haXJpZSBkZSBNb250cGVsbGllcjEbMBkGA1UECxMSNDY1\nMTQ2MTU0MzY1MTQzNjUxMR4wHAYDVQQDDBV0aW50aW5AbW9udHBlbGxpZXIuZnIw\nWTATBgcqhkjOPQIBBggqhkjOPQMBBwNCAAQaftZAwLtg2wVwAT6eGxImMHa84ORV\nVaZuvMQVUSYKAN1BPMNeinZfq1jno3mnQk8te0N0xY2XJ0U5t0XmdJBjo4GkMIGh\nMBMGA1UdJQQMMAoGCCsGAQUFBwMCMAwGA1UdEwEB/wQCMAAwHQYDVR0OBBYEFFWf\ng4ZQdkT+USEEfaPoA93ez6z1MB8GA1UdIwQYMBaAFDEJJIgEBvwoATkzDg8jKGbM\nXwzBMAsGA1UdEQQEMAKCADAvBgNVHR8EKDAmMCSgIqAghh5odHRwOi8vcGtpLmV4\nYW1wbGUub3JnL2NybC5wZW0wDQYJKoZIhvcNAQELBQADggEBAJYVafocwVdBIcL/\np2kSOqczkJvIkW9VdYOAFT/PnuWBjwsyl25VCgkS74Jx56DKlObcaBg2l8tUhD2r\nGfWFcg+Fx1KD34j78wD6Df6OqEE6xRgOt3V8XEzdjU+I0YVTq19uvuiayvu3idqo\nZMT9PIy1A3O5mSSIUS8vCZqXl7/BLV40wK0CjV05HWFxNriNwQxCt+zppsJeTs2Z\nN94hI9qIi3DSHnf7yAwbwSY2gsEmIhjYdyqQsr/Qz1KZoBYvRoHYztRy8EMHjZ9s\nJb6hBk3rDLnQQ5bLdnmBZuQ3nYndcqpZxMw8MSlCsl7UO0QdBIDzxk0Zy8nAzRcN\n1OkqdkI=\n-----END CERTIFICATE-----\n",
    "certificate_request": "-----BEGIN CERTIFICATE REQUEST-----\nMIIBbjCCARMCAQAwgZIxCzAJBgNVBAYTAkZSMRAwDgYDVQQIEwdIZXJhdWx0MRQw\nEgYDVQQHEwtNb250cGVsbGllcjEeMBwGA1UEChMVTWFpcmllIGRlIE1vbnRwZWxs\naWVyMRswGQYDVQQLExI0NjUxNDYxNTQzNjUxNDM2NTExHjAcBgNVBAMMFXRpbnRp\nbkBtb250cGVsbGllci5mcjBZMBMGByqGSM49AgEGCCqGSM49AwEHA0IABBp+1kDA\nu2DbBXABPp4bEiYwdrzg5FVVpm68xBVRJgoA3UE8w16Kdl+rWOejeadCTy17Q3TF\njZcnRTm3ReZ0kGOgHjAcBgkqhkiG9w0BCQ4xDzANMAsGA1UdEQQEMAKCADAKBggq\nhkjOPQQDAgNJADBGAiEArbftUoLBYAwJyHtyluqJc67ypp9WXEYmRcwUplnB4dkC\nIQCIk3mW/NZyOPPBz7EdN8jg0B62ja/PPloKs9em6lkvbA==\n-----END CERTIFICATE REQUEST-----\n",
    "private_key": "-----BEGIN EC PRIVATE KEY-----\nMHcCAQEEIM2dopY0v5llKZoGGaZ84fqm5Z1nKBOILKtOjQxg0fwgoAoGCCqGSM49\nAwEHoUQDQgAEGn7WQMC7YNsFcAE+nhsSJjB2vODkVVWmbrzEFVEmCgDdQTzDXop2\nX6tY56N5p0JPLXtDdMWNlydFObdF5nSQYw==\n-----END EC PRIVATE KEY-----\n",
    "sums": {
      "certificate": {
        "md5": "C8D3EEE26E522C7B26604D9058A708B1",
        "sha-1": "F0AE6E58262CF68AAAFC365BCA65109739DEF716"
      },
      "certificate_request": {
        "md5": "54428C57543C4E66D1D5EEF16DD58D7E",
        "sha-1": "83E6EEFEB0CE41267CB53659051681540BF36D86"
      }
    }
  },
  "errors": [],
  "messages": []
}
```